package com.fr.plugin.form.widget.core;

import com.fanruan.api.cal.ParameterKit;
import com.fanruan.api.xml.XmlKit;
import com.fr.json.JSONArray;
import com.fanruan.api.err.JSONException;
import com.fr.json.JSONObject;
import com.fr.script.Calculator;
import com.fanruan.api.util.ArrayKit;
import com.fr.stable.DependenceProvider;
import com.fr.stable.ParameterProvider;
import com.fanruan.api.err.UtilEvalError;
import com.fr.stable.script.CalculatorProvider;
import com.fr.stable.script.NameSpace;
import com.fr.stable.web.SessionProvider;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLReadable;
import com.fr.stable.xml.XMLable;
import com.fr.stable.xml.XMLableReader;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by richie on 15/12/2.
 */
public class RHIframeAttr implements XMLable, DependenceProvider  {
    public static final String XML_TAG = "RHIframeAttr";

    private RHIframeSource source;

    private ParameterProvider[] parameters;

    public RHIframeSource getSource() {
        return source;
    }

    public void setSource(RHIframeSource source) {
        this.source = source;
    }

    public ParameterProvider[] getParameters() {
        return parameters;
    }

    public void setParameters(ParameterProvider[] parameters) {
        this.parameters = parameters;
    }

    @Override
    public String[] dependence(CalculatorProvider ca) {
        if (source == null) {
            return ArrayKit.EMPTY_STRING_ARRAY;
        }
        List<String> data = new ArrayList<String>();
        data.addAll(Arrays.asList(source.dependence(ca)));
        if (parameters != null) {
            for (ParameterProvider parameter : parameters) {
                data.addAll(Arrays.asList(parameter.dependence(ca)));
            }
        }
        return data.toArray(new String[data.size()]);
    }

    public void mixConfig(JSONObject jo, Calculator c, HttpServletRequest req) throws JSONException {
        if (source != null) {
            jo.put("src", source.getCalculatedUrl(c, req));
            String[] dependence = dependence(c);
            if (!ArrayKit.isEmpty(dependence)) {
                jo.put("dependence", dependence);
                jo.put("valueDependence", dependence);
            }
            jo.put("sourceType", source.getSourceType());
        }
    }

    public void mixJSONData(JSONArray ja, SessionProvider sessionIDInfor, Calculator c) throws JSONException, UtilEvalError {
        NameSpace ns = ParameterKit.createParameterMapNameSpace(sessionIDInfor.getParameterMap());
        c.pushNameSpace(ns);

        if (source != null) {
            try {
                source.mixCalculatedParameters(c, ja, parameters);
            } catch (Exception e) {
            }
        }
           }



    @Override
    public void readXML(XMLableReader reader) {
        if (reader.isChildNode()) {
            String tmpName = reader.getTagName();
            if (ParameterProvider.ARRAY_XML_TAG.equals(tmpName)) {
                //读取Parameters.
                final List<ParameterProvider> tmpParameterList = new ArrayList<ParameterProvider>();

                reader.readXMLObject(new XMLReadable() {
                    @Override
                    public void readXML(XMLableReader reader) {
                        if (ParameterProvider.XML_TAG.equals(reader.getTagName())) {
                            tmpParameterList.add(XmlKit.readParameter(reader));
                        }
                    }
                });

                //转换数组.
                if (!tmpParameterList.isEmpty()) {
                    this.parameters = tmpParameterList.toArray(new ParameterProvider[tmpParameterList.size()]);
                }
            } else if (tmpName.equals(RHIframeSource.XML_TAG)) {
                this.source = (RHIframeSource) XmlKit.readXMLable(reader);
            }
        }
    }

    @Override
    public void writeXML(XMLPrintWriter writer) {
        if (source != null) {
            XmlKit.writeXMLable(writer, source, RHIframeSource.XML_TAG);
        }
        XmlKit.writeParameters(writer, parameters);
    }

    @Override
    public Object clone() throws CloneNotSupportedException {
        RHIframeAttr cloned = (RHIframeAttr)super.clone();
        cloned.source = source;
        cloned.parameters = parameters;
        return cloned;
    }

}