package com.fr.plugin.form.widget.core;

import com.fanruan.api.err.UtilEvalError;
import com.fanruan.api.cal.ParameterKit;
import com.fanruan.api.net.NetworkKit;
import com.fanruan.api.util.RenderKit;
import com.fr.stable.ArrayProvider;
import com.fr.json.JSONArray;
import com.fanruan.api.err.JSONException;
import com.fr.json.JSONObject;
import com.fr.plugin.form.widget.URLUtils;
import com.fr.script.Calculator;
import com.fr.stable.FormulaProvider;
import com.fanruan.api.util.CodeKit;
import com.fr.stable.ParameterProvider;
import com.fanruan.api.macro.ProductConstants;
import com.fanruan.api.util.StringKit;
import com.fanruan.api.report.form.WidgetName;
import com.fr.stable.script.CalculatorProvider;
import com.fr.stable.xml.XMLPrintWriter;
import com.fr.stable.xml.XMLableReader;
import com.fr.third.org.apache.http.NameValuePair;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Created by richie on 15/12/2.
 */
public class TemplateSource extends RHIframeSource {


    private String path;

    public TemplateSource() {

    }

    public TemplateSource(String path) {
        this.path = path;
    }

    public String getPath() {
        return path;
    }

    public void setPath(String path) {
        this.path = path;
    }

    @Override
    public String getCalculatedUrl(Calculator calculator, HttpServletRequest req) {
        try {
            String realPath = RenderKit.render(path, calculator);
            List<NameValuePair> pairs = URLUtils.parse("__emb__=" + realPath);
            List<NameValuePair> extraPairs = new ArrayList<NameValuePair>();
            for (NameValuePair pair : pairs) {
                if ("__emb__".equals(pair.getName())) {
                    realPath = pair.getValue();
                } else {
                    extraPairs.add(pair);
                }
            }

            if (realPath.contains(".cpt")) {
                if (isNewerThan9()) {
                    return NetworkKit.createServletURL(req) + "/view/report?viewlet=" + CodeKit.encodeURIComponent(realPath) + createQueryString(extraPairs);
                } else {
                    return NetworkKit.createServletURL(req) + "?reportlet=" + CodeKit.encodeURIComponent(realPath) + createQueryString(extraPairs);
                }
            } else if (realPath.contains(".frm")) {
                if (isNewerThan9()) {
                    return NetworkKit.createServletURL(req) + "/view/form?viewlet=" + CodeKit.encodeURIComponent(realPath) + createQueryString(extraPairs);
                }
                return NetworkKit.createServletURL(req) + "?formlet=" + CodeKit.encodeURIComponent(realPath) + createQueryString(extraPairs);
            }
        } catch (Exception e) {
            return null;
        }
        return null;
    }

    private boolean isNewerThan9() {
        return Integer.parseInt(ProductConstants.MAIN_VERSION) > 9;
    }

    private String createQueryString(List<NameValuePair> pairs) {
        StringBuilder sb = new StringBuilder();
        for (int i = 0, len = pairs.size(); i < len; i++) {
            NameValuePair pair = pairs.get(i);
            sb.append("&");
            String key = pair.getName();
            if (key == null) {
                key = StringKit.EMPTY;
            }
            sb.append(CodeKit.encodeURIComponent(key));
            sb.append("=");
            String value = pair.getValue();
            if (value == null) {
                value = StringKit.EMPTY;
            }
            sb.append(CodeKit.encodeURIComponent(value));
        }
        return sb.toString();
    }

    @Override
    public String[] dependence(CalculatorProvider ca) {
        ParameterProvider[] parameters = ParameterKit.analyze4Parameters(path, false);
        List<String> collections = new ArrayList<String>();
        for (ParameterProvider parameter : parameters) {
            collections.addAll(Arrays.asList(parameter.dependence(ca)));
        }
        return collections.toArray(new String[collections.size()]);
    }

    @Override
    public void mixCalculatedParameters(Calculator c, JSONArray ja, ParameterProvider[] parameters) throws JSONException, UtilEvalError {
        if (parameters == null) {
            return;
        }
        // richie：这里的key/value不需要做CJKEncode，在前端会处理的
        for (ParameterProvider parameter : parameters) {
            JSONObject jo = JSONObject.create();
            String key = parameter.getName();
            Object obj = parameter.getValue();
            if (obj instanceof FormulaProvider) {
                try{
                    String content = ((FormulaProvider) obj).getContent();
                    obj = c.evalValue(content);
                }catch (Exception e) {
                }
            }
            if (obj instanceof String) {
                jo.put(parameter.getName(), obj);
            } else if (obj instanceof ArrayProvider) {
                jo.put(key, obj);
            } else if (obj instanceof WidgetName) {
                jo.put("widgetName", ((WidgetName) obj).getName());
            } else {
                // 还可以是数字啊什么的
                jo.put(key, obj);
            }
            ja.put(jo);
        }
    }

    @Override
    public String getSourceType() {
        return "tpl";
    }

    @Override
    public String toString() {
        return path;
    }

    @Override
    public void readXML(XMLableReader reader) {
        super.readXML(reader);
        if (reader.isChildNode()) {
            String nodeName = reader.getTagName();
            if ("Attr".equals(nodeName)) {
                this.path = reader.getAttrAsString("path", null);
            }
        }
    }

    @Override
    public void writeXML(XMLPrintWriter writer) {
        super.writeXML(writer);
        writer.startTAG("Attr");
        writer.attr("path", path);
        writer.end();
    }
}